@@ -24,12 +24,32 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploaderConfig.h 554 2005-09-01 11:49:36Z svn $
+ * $Id: UploaderConfig.h 573 2005-09-03 07:50:04Z svn $
  *****************************************************************************/
 
 #ifndef UPLOADERCONFIG_H
 #define UPLOADERCONFIG_H
 
+#define APR_NOT_DONE_YET 0
+
+#include "httpd.h"
+#include "apr_pools.h"
+#include "apr_time.h"
+#include "apr_global_mutex.h"
+#ifdef LOCK_TYPE_MUTEX
+#include "apr_thread_mutex.h"
+#else
+#include "apr_thread_rwlock.h"
+#endif
+
+#undef strtoul
+
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_NAME
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+#undef PACKAGE_VERSION
+
 #include "RFC1867ParserImpl.h"
 #include "UploadItemWriter.h"
 #include "UploadItemReader.h"
@@ -38,14 +58,6 @@
 #include "TemplateParser.h"
 #include "TemplateExecutor.h"
 
-#include "apr_pools.h"
-#include "apr_time.h"
-#include "apr_global_mutex.h"
-#ifdef LOCK_TYPE_MUTEX
-#include "apr_thread_mutex.h"
-#else
-#include "apr_thread_rwlock.h"
-#endif
 
 #include <cstdlib>
 
@@ -73,31 +85,10 @@ public:
         }
     } page_template;
 
-    // ݤΤ public
-    const char *url;
-    const char *file_dir;
-    const char *thumb_dir;
-    const char *tmp_dir;
-    const char *lock_dir;
-    const char *view_tmpl_path;
-    const char *download_tmpl_path;
-    const char *thumb_tmpl_path;
-    const char *error_tmpl_path;
-    apr_uint64_t total_file_size_limit;
-    apr_size_t total_file_number_limit;
-    apr_size_t max_file_size;
-    apr_size_t per_page_item_number;
-    apr_size_t remove_item_number;
-    apr_global_mutex_t *glock;
-
-    UploadItemList *item_list;
-    TemplateExecutor *texecutor;
-    UploadParser *uparser;
-
     UploaderConfig();
     ~UploaderConfig();
 
-    bool init(apr_pool_t *pool, apr_pool_t *tmp_pool);
+    void init(request_rec *r);
     void finalize();
 
     page_template *get_view_template(apr_pool_t *pool);
@@ -107,10 +98,30 @@ public:
 
     UploadItemReader *get_ureader(apr_pool_t *pool);
     UploadItemWriter *get_uwriter(apr_pool_t *pool);
-    apr_status_t read_lock();
-    apr_status_t write_lock();
-    apr_status_t read_unlock();
-    apr_status_t write_unlock();
+    void read_lock();
+    void read_unlock();
+    void gwrite_lock();
+    void gwrite_unlock();
+
+    // ݤΤ public
+    const char *url;
+    const char *file_dir;
+    const char *thumb_dir;
+    const char *tmp_dir;
+    const char *magick_dir;
+    const char *view_tmpl_path;
+    const char *download_tmpl_path;
+    const char *thumb_tmpl_path;
+    const char *error_tmpl_path;
+    apr_uint64_t total_file_size_limit;
+    apr_size_t total_file_number_limit;
+    apr_size_t max_file_size;
+    apr_size_t per_page_item_number;
+    apr_size_t remove_item_number;
+
+    UploadItemList *item_list;
+    TemplateExecutor *texecutor;
+    UploadParser *uparser;
 
 private:
     static const apr_uint64_t TOTAL_FILE_SIZE_LIMIT;
@@ -121,35 +132,42 @@ private:
     static const apr_size_t PER_PAGE_ITEM_NUMBER;
     static const apr_size_t ITEM_RELOAD_THRESHOLD;
 
-    page_template *view_tmpl;
-    apr_pool_t *view_tmpl_pool;
-    page_template *download_tmpl;
-    apr_pool_t *download_tmpl_pool;
-    page_template *thumb_tmpl;
-    apr_pool_t *thumb_tmpl_pool;
-    page_template *error_tmpl;
-    apr_pool_t *error_tmpl_pool;
-    apr_pool_t *list_pool;
-#if APR_HAS_THREADS
-#ifdef LOCK_TYPE_MUTEX
-    apr_thread_mutex_t *lock;
-#else
-    apr_thread_rwlock_t *lock;
-#endif
-#endif
-    volatile bool is_init;
-
-    void exec_init(apr_pool_t *pool, apr_pool_t *tmpl_pool);
+    void exec_init();
     void validate();
 #if APR_HAS_THREADS
-    void init_lock(apr_pool_t *pool, apr_pool_t *tmp_pool);
+    void init_lock();
 #endif
-    void load_list(apr_pool_t *pool);
-    void update_list(apr_pool_t *pool, apr_pool_t *tmp_pool);
+    void load_list();
+    void update_list();
     void load_template(apr_pool_t *pool, apr_pool_t **tmpl_pool,
                        const char *template_path, page_template *tmpl);
 
+    void write_lock();
+    void write_unlock();
+    void global_lock();
+    void global_unlock();
+
     static apr_time_t get_mtime(apr_pool_t *pool, const char *path);
+
+    apr_pool_t *pool_;
+    page_template *view_tmpl_;
+    apr_pool_t *view_tmpl_pool_;
+    page_template *download_tmpl_;
+    apr_pool_t *download_tmpl_pool_;
+    page_template *thumb_tmpl_;
+    apr_pool_t *thumb_tmpl_pool_;
+    page_template *error_tmpl_;
+    apr_pool_t *error_tmpl_pool_;
+    apr_pool_t *list_pool_;
+    apr_global_mutex_t *glock_;
+#if APR_HAS_THREADS
+#ifdef LOCK_TYPE_MUTEX
+    apr_thread_mutex_t *lock_;
+#else
+    apr_thread_rwlock_t *lock_;
+#endif
+#endif
+    volatile bool is_init_;
 };
 
 #endif
