@@ -24,7 +24,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: ApachePostReader.cpp 557 2005-09-01 14:15:15Z svn $
+ * $Id: ApachePostReader.cpp 563 2005-09-02 03:22:28Z svn $
  *****************************************************************************/
 
 #include "ApachePostReader.h"
@@ -70,10 +70,11 @@ void ApachePostReader::read(char *buffer, apr_size_t size, apr_size_t *read_size
         get_brigade();
     }
 
-    if (APR_BUCKET_IS_EOS(bucket_)) {
+    if (UNLIKELY(APR_BUCKET_IS_EOS(bucket_))) {
         is_eos_ = true;
         return;
-    } else if (APR_BUCKET_IS_FLUSH(bucket_)) {
+    }
+    if (APR_BUCKET_IS_FLUSH(bucket_)) {
         bucket_ = APR_BUCKET_NEXT(bucket_);
         return read(buffer, size, read_size);
     }
