@@ -24,14 +24,11 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemList.cpp 552 2005-09-01 08:46:11Z svn $
+ * $Id: UploadItemList.cpp 570 2005-09-02 21:21:36Z svn $
  *****************************************************************************/
 
 #include "UploadItemList.h"
 #include "TemplateVariableCreator.h"
-#ifdef MAKE_THUMBNAIL
-#include "ThumbnailWriter.h"
-#endif
 #include "Auxiliary.h"
 #include "Misc.h"
 
@@ -63,7 +60,10 @@ UploadItemList::UploadItemList(apr_pool_t *pool,
       thumb_dir_(thumb_dir),
       total_number_limit_(total_number_limit),
       total_size_limit_(total_size_limit),
-      reader(pool, file_dir),
+      reader_(pool, file_dir),
+#ifdef MAKE_THUMBNAIL
+      twriter_(pool, file_dir, thumb_dir),
+#endif
       total_size_(0),
       mtime_(0)
 {
@@ -76,7 +76,7 @@ void UploadItemList::add(const char *file_path, bool is_new)
     item_info *info;
     apr_file_t *file;
 
-    header = reader.read(file_path, &file);
+    header = reader_.read(file_path, &file);
     apr_file_close(file);
 
     // pool_ ʤ data_pool_ ȤȤޤ
@@ -122,7 +122,7 @@ apr_time_t UploadItemList::get_mtime() const
     return mtime_;
 }
 
-void UploadItemList::set_mtime()
+void UploadItemList::update_mtime()
 {
     mtime_ = mtime(pool_, file_dir_);
 }
@@ -136,8 +136,6 @@ void UploadItemList::remove(const char *file_path, const char *password)
 {
     item_iterator i;
 
-    load_file();
-
     for (i = begin(); i != end(); i++) {
         if (strcmp((*i)->file_path, file_path) == 0) {
             break;
@@ -155,7 +153,7 @@ void UploadItemList::remove(const char *file_path, const char *password)
     remove(*i);
 
     write_cache();
-    set_mtime();
+    update_mtime();
 }
 
 TemplateExecutor::variable *UploadItemList::to_varray(apr_pool_t *pool,
@@ -270,12 +268,12 @@ void UploadItemList::init()
     load_file();
     load_thumb();
 
-    set_mtime();
+    update_mtime();
 }
 
 bool UploadItemList::load_file()
 {
-    ilist_.resize(0);
+    ilist_.clear();
     total_size_ = 0;
 
     if (data_pool_ != NULL) {
@@ -342,6 +340,11 @@ void UploadItemList::load_file_by_item()
 
 void UploadItemList::load_file_by_cache()
 {
+    char *path;
+    UploadItem::header *header;
+    item_info *info;
+    apr_size_t number;
+    apr_size_t i;
     apr_pool_t *tmp_pool;
 
     if (apr_pool_create(&tmp_pool, pool_) != APR_SUCCESS) {
@@ -349,34 +352,28 @@ void UploadItemList::load_file_by_cache()
     }
 
     try {
-        char *path;
-        UploadItem::header *header;
-        item_info *info;
-        apr_size_t number;
-        apr_size_t i;
-
         read_cache(tmp_pool, &path, &header, &number);
 
-        for (i = 0; i < number; i++) {
-            APR_PALLOC(info, item_info *, data_pool_, sizeof(item_info));
-
-            info->file_path = path;
-            info->header = header;
-
-            add(info);
-
-            path += strlen(path) + 1; // 
-            header++;
-        }
-
+        apr_pool_destroy(tmp_pool);
+    } catch(const char *message) {
         apr_pool_destroy(tmp_pool);
 
-        sort();
-    } catch(const char *) {
-        apr_pool_destroy(tmp_pool);
-
-        throw;
+        throw message;
     }
+
+    for (i = 0; i < number; i++) {
+        APR_PALLOC(info, item_info *, data_pool_, sizeof(item_info));
+
+        info->file_path = path;
+        info->header = header;
+
+        add(info);
+
+        path += strlen(path) + 1; // 
+        header++;
+    }
+
+    sort();
 }
 
 void UploadItemList::load_thumb()
@@ -417,12 +414,9 @@ void UploadItemList::load_thumb()
 
 void UploadItemList::add_new(item_info *info)
 {
-    bool is_cache_used;
-
 #ifdef MAKE_THUMBNAIL
     try {
-        ThumbnailWriter twriter(pool_, file_dir_, thumb_dir_);
-        twriter.write(info->file_path);
+        twriter_.write(info->file_path);
 
         tlist_.push_front(info->file_path);
     } catch(const char *) {
@@ -430,10 +424,7 @@ void UploadItemList::add_new(item_info *info)
     }
 #endif
 
-    is_cache_used = load_file();
-    if (is_cache_used) {
-        add(info);
-    }
+    add(info);
 
     while (ilist_.size() > total_number_limit_) {
         remove(ilist_.back());
@@ -443,7 +434,7 @@ void UploadItemList::add_new(item_info *info)
     }
 
     write_cache();
-    set_mtime();
+    update_mtime();
 }
 
 UploadItemList::item_iterator UploadItemList::begin() const
