@@ -24,11 +24,14 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: mod_uploader.cpp 574 2005-09-03 10:14:13Z svn $
+ * $Id: mod_uploader.cpp 589 2005-09-05 02:01:25Z svn $
  *****************************************************************************/
 
+#define __STDC_CONSTANT_MACROS
+
 #include "mod_uploader.h"
 
+#include "UploaderConfig.h"
 #include "UploadItemCreator.h"
 #include "TemplateVariableCreator.h"
 #include "RssView.h"
@@ -40,6 +43,17 @@
 #include "Misc.h"
 
 #include "http_core.h"
+#if !defined(OS2) && !defined(WIN32) && !defined(BEOS) && !defined(NETWARE)
+#ifdef __cplusplus
+extern "C" {
+#endif
+#include "unixd.h"
+#ifdef __cplusplus
+}
+#endif
+#define MOD_UPLOADER_SET_MUTEX_PERMS
+#endif
+
 #define APR_WANT_STRFUNC
 #include "apr_want.h"
 
@@ -47,19 +61,12 @@
 
 #include <memory>
 
-#define MATCH(str, pat) (strstr(str, pat) != 0)
 
-#ifdef DEBUG
-#define SERROR(format, str) \
-    fprintf(stderr, format, str); fputc('\n', stderr)
-#define RERROR(format, str) \
-    fprintf(stderr, format, str); fputc('\n', stderr)
-#else
-#define SERROR(format, str) \
-    ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "[mod_uploader] " format, str)
-#define RERROR(format, str) \
-    ap_log_rerror(APLOG_MARK, APLOG_CRIT, 0, r, "[mod_uploader] " format, str)
-#endif
+typedef UploaderConfig                      uconfig;
+typedef UploaderConfig::page_template       page_template;
+typedef TemplateExecutor::variable          variable;
+typedef TemplateExecutor::scalar            scalar;
+typedef TemplateExecutor::variable_map      variable_map;
 
 static int command_handler(request_rec *r, uconfig *config, const char *arg);
 static int view(request_rec *r, uconfig *config, const char *arg);
@@ -86,11 +93,33 @@ static variable_map *get_thumb_vmap(apr_pool_t *pool, UploaderConfig *config,
                                     page_template *tmpl, apr_size_t page);
 #endif
 static apr_size_t get_page(request_rec *r);
+
+static const char *get_poster_address(request_rec *r);
+static bool is_poster_list_contain(request_rec *r, const char *address);
+static void add_poster_list(request_rec *r, const char *address);
+static int poster_compare(const poster *a, const poster *b);
+
 static const char *get_name_encoding(request_rec *r);
 
+#define MATCH(str, pat) (strstr(str, pat) != 0)
+
+#ifdef DEBUG
+#define SERROR(format, str) \
+    fprintf(stderr, "[" PACKAGE_NAME "] " format, str); fputc('\n', stderr)
+#define RERROR(format, str) \
+    fprintf(stderr, "[" PACKAGE_NAME "] " format, str); fputc('\n', stderr)
+#else
+#define SERROR(format, str) \
+    ap_log_error(APLOG_MARK, APLOG_CRIT, 0, s, "[" PACKAGE_NAME "] " format, str)
+#define RERROR(format, str) \
+    ap_log_rerror(APLOG_MARK, APLOG_CRIT, 0, r, "[" PACKAGE_NAME "] " format, str)
+#endif
+
 
 int command_handler(request_rec *r, uconfig *config, const char *arg)
 {
+    const char *command;
+
     if (*arg == '\0') {
         apr_table_set(r->headers_out, "Location", apr_pstrcat(r->pool, r->uri, "/", NULL));
 
@@ -98,7 +127,7 @@ int command_handler(request_rec *r, uconfig *config, const char *arg)
     }
     arg++;
 
-    const char *command = ap_getword(r->pool, &arg, '/');
+    command = ap_getword(r->pool, &arg, '/');
 
     if (*command == '\0') {
         return view(r, config, arg);
@@ -248,6 +277,7 @@ int rss(request_rec *r, uconfig *config, const char *arg)
 
 int upload(request_rec *r, uconfig *config, const char *arg)
 {
+    const char *address;
     apr_status_t status;
 
     if (r->method_number != M_POST) {
@@ -260,6 +290,12 @@ int upload(request_rec *r, uconfig *config, const char *arg)
         return HTTP_NO_CONTENT;
     }
 
+    address = get_poster_address(r);
+    if (is_poster_list_contain(r, address)) {
+        return error(r, config,
+                     apr_psprintf(r->pool, "%u ô֤ϺƤǤޤ", POST_INTERVAL_SEC));
+    }
+
     try {
         UploadItem::header *header;
         const char *tmp_path;
@@ -279,15 +315,17 @@ int upload(request_rec *r, uconfig *config, const char *arg)
         auto_ptr<UploadItemWriter> writer(config->get_uwriter(r->pool));
         file_name = writer->write(header, tmp_path);
 
+        config->gwrite_lock();
         try {
-            config->gwrite_lock();
             config->item_list->add(file_name, true);
             config->gwrite_unlock();
         } catch(const char *) {
-            config->read_unlock();
+            config->gwrite_unlock();
             throw;
         }
 
+        add_poster_list(r, address);
+
         return redirect_top(r);
     } catch(const char *message) {
         return error(r, config, message);
@@ -415,8 +453,8 @@ int remove(request_rec *r, uconfig *config, const char *arg)
             return error(r, config, "եޤϥѥɤꤵƤޤ");
         }
 
+        config->gwrite_lock();
         try {
-            config->gwrite_lock();
             config->item_list->remove(file, password);
             config->remove_item_number++;
             config->gwrite_unlock();
@@ -456,6 +494,8 @@ int error(request_rec *r, uconfig *config, const char *error)
 
 int redirect_top(request_rec *r)
 {
+    // 쥯ȤϻȤʤ
+
     ap_set_content_type(r, "text/html");
 
     ap_rputs("<?xml version=\"1.0\" encoding=\"EUC-JP\"?>", r);
@@ -464,7 +504,7 @@ int redirect_top(request_rec *r)
     ap_rputs(" <head>", r);
     ap_rputs("  <meta http-equiv=\"content-type\" content=\"application/xhtml+xml\" />", r);
     ap_rputs("  <meta http-equiv=\"refresh\" content=\"0;url=../\" />", r);
-    ap_rputs("  <title>mod_uploader</title>", r);
+    ap_rputs("  <title>" PACKAGE_NAME "</title>", r);
     ap_rputs(" </head>", r);
     ap_rputs(" <body />", r);
     ap_rputs("</html>", r);
@@ -650,6 +690,74 @@ apr_size_t get_page(request_rec *r)
     return static_cast<apr_size_t>(page);
 }
 
+const char *get_poster_address(request_rec *r)
+{
+    char *address = "";
+
+    apr_sockaddr_ip_get(&address, r->connection->remote_addr);
+
+    return address;
+}
+
+bool is_poster_list_contain(request_rec *r, const char *address)
+{
+    sconfig *config;
+    apr_time_t now;
+    apr_size_t i;
+
+    config = static_cast<sconfig *>(ap_get_module_config(r->server->module_config,
+                                                         &uploader_module));
+
+    now = apr_time_now();
+    for (i = 0; i < POSTER_HISTORY_NUMBER; i++) {
+        if ((now - config->poster_list[i].time) > (POST_INTERVAL_SEC*APR_USEC_PER_SEC)) {
+            break;
+        }
+
+        if (strcmp(config->poster_list[i].address, address) == 0) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+int poster_compare(const poster *a, const poster *b)
+{
+    return b->time > a->time;
+}
+
+void add_poster_list(request_rec *r, const char *address)
+{
+    sconfig *config;
+    apr_time_t now;
+    apr_size_t i, j;
+
+    config = static_cast<sconfig *>(ap_get_module_config(r->server->module_config,
+                                                         &uploader_module));
+
+    now = apr_time_now();
+    for (i = 0; i < POSTER_HISTORY_NUMBER; i++) {
+        if ((now - config->poster_list[i].time) > (POST_INTERVAL_SEC*APR_USEC_PER_SEC)) {
+            break;
+        }
+    }
+
+    if (i == POSTER_HISTORY_NUMBER) {
+        i--;
+    }
+    config->poster_list[i].time = now;
+    strncpy(config->poster_list[i].address, address, sizeof(config->poster_list[i].address));
+    j = i;
+
+    while (++i < POSTER_HISTORY_NUMBER) {
+        config->poster_list[i].time = 0;
+    }
+
+    qsort(config->poster_list, j + 1, sizeof(poster),
+          reinterpret_cast<int(*)(const void *, const void *)>(poster_compare));
+}
+
 const char *get_name_encoding(request_rec *r)
 {
     const char *agent = apr_table_get(r->headers_in, "User-Agent");
@@ -686,6 +794,20 @@ static int uploader_handler(request_rec *r)
     return command_handler(r, config, r->path_info);
 }
 
+static apr_status_t finalize_server_config(void *data)
+{
+    sconfig *config;
+
+    config = static_cast<sconfig *>(data);
+
+    if (config->poster_list_shm != NULL) {
+        apr_shm_destroy(config->poster_list_shm);
+        config->poster_list_shm = NULL;
+    }
+
+    return APR_SUCCESS;
+}
+
 static apr_status_t finalize_dir_config(void *data)
 {
     uconfig *config;
@@ -704,8 +826,13 @@ static void *create_server_config(apr_pool_t *p, server_rec *s)
     config = static_cast<sconfig *>(apr_palloc(p, sizeof(sconfig)));
 
     apr_temp_dir_get(&tmp_dir, p);
+
     apr_filepath_merge(&(config->glock_path), tmp_dir, GLOCK_NAME,
                        APR_FILEPATH_NOTABOVEROOT, p);
+    apr_filepath_merge(&(config->poster_list_path), tmp_dir, POSTER_LIST_NAME,
+                       APR_FILEPATH_NOTABOVEROOT, p);
+
+    config->poster_list_shm = NULL;
 
     return config;
 }
@@ -726,17 +853,57 @@ static int post_config(apr_pool_t *pconf, apr_pool_t *plog,
                        apr_pool_t *ptemp, server_rec *s)
 {
     sconfig *config;
+    void *user_data;
+    apr_status_t status;
+    char error[1024];
+
+    apr_pool_userdata_get(&user_data, USER_DATA_KEY, s->process->pool);
+    if (user_data == NULL) {
+        apr_pool_userdata_set(reinterpret_cast<const void *>(1), USER_DATA_KEY,
+                              apr_pool_cleanup_null, s->process->pool);
+        return OK;
+    }
 
     config = static_cast<sconfig *>(ap_get_module_config(s->module_config, &uploader_module));
 
-    if (apr_global_mutex_create(&(config->glock), config->glock_path,
-                                APR_LOCK_DEFAULT, pconf) != APR_SUCCESS) {
+    status = apr_global_mutex_create(&(config->glock), config->glock_path,
+                                     APR_LOCK_DEFAULT, pconf);
+    if (status != APR_SUCCESS) {
         SERROR("Can not create global mutex (%s).", config->glock_path);
+        SERROR("(%s)", apr_strerror(status, error, sizeof(error)));
 
         return HTTP_INTERNAL_SERVER_ERROR;
     }
 
-    // Windows Ǥ򤹤ȥӥ̾ѤäƤޤ餷
+#ifdef MOD_UPLOADER_SET_MUTEX_PERMS
+    status = unixd_set_global_mutex_perms(config->glock);
+    if (status != APR_SUCCESS) {
+        SERROR("Can not set permissions on global mutex (%s).", config->poster_list_path);
+        SERROR("(%s)", apr_strerror(status, error, sizeof(error)));
+
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+#endif
+    apr_pool_cleanup_register(pconf, config, finalize_server_config, apr_pool_cleanup_null);
+
+    // ޤϡAnonymous Shared Memory Ƥߤ롥
+    status = apr_shm_create(&(config->poster_list_shm), sizeof(poster)*POSTER_HISTORY_NUMBER,
+                            NULL, pconf);
+    if (status == APR_ENOTIMPL) {
+        status = apr_shm_create(&(config->poster_list_shm), sizeof(poster)*POSTER_HISTORY_NUMBER,
+                                config->poster_list_path, pconf);
+    }
+    if (status != APR_SUCCESS) {
+        SERROR("Can not create shared segment file (%s).", config->poster_list_path);
+        SERROR("(%s)", apr_strerror(status, error, sizeof(error)));
+
+        return HTTP_INTERNAL_SERVER_ERROR;
+    }
+
+    config->poster_list = static_cast<poster *>(apr_shm_baseaddr_get(config->poster_list_shm));
+    memset(config->poster_list, 0, sizeof(poster)*POSTER_HISTORY_NUMBER);
+
+    // windows Ǥ򤹤ȥӥ̾ѤäƤޤ餷
 #ifndef WIN32
     ap_add_version_component(pconf, PACKAGE_NAME "/" PACKAGE_VERSION);
 #endif
@@ -756,6 +923,17 @@ static void child_init(apr_pool_t *p, server_rec *s)
 
         return;
     }
+
+    if (!config->poster_list_shm) {
+        if (apr_shm_attach(&(config->poster_list_shm), config->poster_list_path,
+                           p) != APR_SUCCESS) {
+            SERROR("Can not attach to shared segment file (%s).", config->poster_list_path);
+
+            return;
+        }
+    }
+
+    config->poster_list = static_cast<poster *>(apr_shm_baseaddr_get(config->poster_list_shm));
 }
 
 static const char *set_url(cmd_parms *parms, void *mconfig, char *arg)
